<?php
/*
Plugin Name: Elo Rating
Plugin URI: http://davidn.de
Description: This plugin provides an elo rating system for wordpress.
Version: 0.2
Author: David Nellessen
Author URI: http://davidn.de
Author Email: info@davidn.de
License: GPL

Copyright 2012 David Nellessen (info@davidn.de)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License, version 2, as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

// Define environement vars.
define('ELO_RATING_VERSION', '0.2');

/**
 * Activation hook for this plugin.
 * 
 * Installation rutine doing:
 * - Register options.
 */
register_activation_hook(__FILE__, 'elo_rating__activation');
function elo_rating__activation() {
  // Save installed version.
  add_option("elo_rating_version", ELO_RATING_VERSION);
  // Set default category for posts to be rated.
  add_option("elo_rating_category", 'elo');
}

/**
 * Implements init action hook.
 * 
 * Load stylesheets and js files.
 */
add_action( 'init', 'elo_rating__init' );
function elo_rating__init(){
  //Internationalization
  //load_plugin_textdomain( 'elo_rating', false, basename(dirname(__FILE__)) .'/languages' );
  //register stylesheets
  $stylesheeturl = plugins_url( '/css/frontend.css' , __FILE__ );
  wp_register_style('elo-rating-frontendstyle', $stylesheeturl);
  wp_enqueue_style( 'elo-rating-frontendstyle');
  //register javascript
  $scripturl = plugins_url( '/js/frontend.js' , __FILE__ );
  wp_register_script('elo-rating-frontendjs', $scripturl, array('jquery'), FALSE, TRUE);
  wp_enqueue_script( 'elo-rating-frontendjs');
  // http://www.garyc40.com/2010/03/5-tips-for-using-ajax-in-wordpress/#js-global
  //wp_localize_script( 'elo-rating-frontendjs', 'MyAjax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );
}
add_action( 'admin_init', 'elo_rating__admin_init', 1 );
function elo_rating__admin_init(){
  //Internationalization
  //load_plugin_textdomain( 'elo_rating', false, basename(dirname(__FILE__)) .'/languages' );
}

/**
 * Add ajax script to the footer of the page.
 * 
 * The js function elo_rating__rate() is triggered when rating. The
 * information about the battle (winning and loosing post) is send to
 * admin-ajax.php and the page gets a refresh (with random query param
 * to avoid caching).
 *
 * @todo improve randomization of url so that it works when parameters are set.
 */
add_action('wp_footer', 'elo_rating__footer');
function elo_rating__footer() {
  echo '
    <script type="text/javascript" >
    function elo_rating__rate(url_liked) {
    	var data = {
    		action: \'elo_rating_rate_callback\',
    		url_liked: url_liked,
    		elo_rating__post_A: elo_rating__post_A,
    		elo_rating__post_B: elo_rating__post_B,
    		security: "' . wp_create_nonce("elo-rating-nonce") . '"
    	};
    
    	// since 2.8 ajaxurl is always defined in the admin header and points to admin-ajax.php
    	jQuery.post("' . admin_url( 'admin-ajax.php' ) . '", data, function(response) {
    		//alert(\'Got this from the server: \' + response);
    		var url = window.location.protocol + "//" + window.location.host + "/" + window.location.pathname;
    		url += "?rand=" + Math.round(Math.random() * 10000);
    		window.location.href = url;
    	});
    };
    </script>
  ';
}

/**
 * Define ajax callbacks.
 *
 * This functions are called the the js function elo_rating__rate() sends
 * data to the server. It parses the data and calls elo_rating__rate() to
 * process the rating logic.
 */
add_action('wp_ajax_elo_rating_rate_callback', 'elo_rating_rate_callback');
add_action('wp_ajax_nopriv_elo_rating_rate_callback', 'elo_rating_rate_callback');
function elo_rating_rate_callback() {
  // Check nounce first for security.
  check_ajax_referer( 'elo-rating-nonce', 'security' );
  global $wpdb; // this is how you get access to the database
  
  // Get post ids that battled each other.
  $url_liked = (string)($_POST['url_liked']);
  $post_A = (int)($_POST['elo_rating__post_A']);
  $post_B = (int)($_POST['elo_rating__post_B']);
  $url_liked = (string)($_POST['url_liked']);
  $post_liked = url_to_postid($url_liked);
  
  // Get win/lose.
  $win_id = $post_A == $post_liked ? $post_A : $post_B;
  $lost_id = $post_A == $post_liked ? $post_B : $post_A;
  
  echo 'Post won: ' . $win_id . '. Post lost: ' . $lost_id . '.';
  elo_rating__rate($win_id, $lost_id);
  
  die(); // this is required to return a proper result
}

/**
 * Add Short for the frontend.
 *
 * The shortcode generates 
 * 
 * @todo 'orderby' should prefer popular items.
 * @todo Introduce shortcode parameter 'cat' for post categories.
 */
add_shortcode( 'elo' , 'elo_rating__frontend' );
function elo_rating__frontend($atts) {
  // Get the permalink to the current post.
  global $post;
  $permalink = get_permalink($post->ID);
  
  $html = '';
  // Add Facebook's JS SDK. @todo There should be a generic integration including compatibility to other plugins.
  $html .= '<div id="fb-root"></div>';
  $html_s .= '<script>(function(d, s, id) {
    var js, fjs = d.getElementsByTagName(s)[0];
    if (d.getElementById(id)) return;
    js = d.createElement(s); js.id = id;
    js.src = "//connect.facebook.net/de_DE/all.js#xfbml=1";
    fjs.parentNode.insertBefore(js, fjs);
  }(document, \'script\', \'facebook-jssdk\'));</script>';
  
  $html .= '
  <script>
  </script>';

  // Start creating output.
  $html .= '<div class="clearfix elo-rating-items">';
  
  // Get the category for which we should rate posts.
  $category = get_option('elo_rating_category', 'elo');
  
  // Create Query.
  $the_query = new WP_Query( array(
    'category_name' => $category,
    'orderby' => 'rand',
    'posts_per_page' => 2,
  ));
  // Store postids
  $posids = array();
  // The Loop
  $i = 0;
  while ( $the_query->have_posts() ) : $the_query->the_post();
  $i++;
  $posids[] = get_the_ID();
  $html .= '<div class="elo-item-container"><div class="elo-item">';
  $html .= '<div class="elo-post"><strong>Candidate ' . $i . ':</strong><h1>' . get_the_title() . '</h1></div>';
  $html .= '<span class="elo-vote-headline">Vote using like button:</span>';
  $html .= '<div class="fb-like" data-href="' . get_permalink() . '" data-send="false" data-layout="box_count" data-width="55" data-show-faces="false" data-font="segoe ui"></div>';
  $html .= '<span class="elo-vote-headline">or without:</span>';
  $html .= '<div><a class="elo-vote-link" onclick="elo_rating__rate(\'' . get_permalink() . '\')">Vote for it</a></div>';
  $html .= '</div></div>';
  // Add "vs" in the middle.
  if ($i==1) {
    $html .= '<div class="elo-vs-container">vs.</div>';
  }
  if($i == 2) break;
  endwhile;
  // Reset Post Data
  wp_reset_postdata();
  
  // Make Post ids available for rating in JS.
  $html .= '<script>';
  for ($i = 0; $i<2; $i++) {
    if(empty($posids[$i])) $posids[$i] = 0;
  }
  $html .= '
    var elo_rating__post_A = ' . $posids[0] . ';
    var elo_rating__post_B = ' . $posids[1] . ';
  ';
  $html .= '</script>';
  
  $html .= '</div>';
  
  // Create rating list.
  $the_query = new WP_Query( array(
    'category_name' => $category,
    'orderby' => 'meta_value_num',
    'posts_per_page' => 20,
    'meta_key' => 'elo_rating__rating'
  ));
  $html .= '<div class="elo-rating-list">
  <h2>Best rated Items</h2>
  <ol>
  ';
  while ( $the_query->have_posts() ) : $the_query->the_post();
  $html .= '<li class="elo-post">' . get_the_title() . '</li>';
  endwhile;
  // Reset Post Data
  wp_reset_postdata();
  $html .= '</ol>';
  $html .= '</div>';
  
  return $html;
}

/**
 * Function to process a rating.
 * 
 * This is used to process a rating, calculate new scores for both posts and
 * store it in meta fields associated to the posts.
 *
 * @param $post_win
 *   Post ID of the post rated.
 * @param $post_lost
 *   Post ID of the post rated against.
 */
function elo_rating__rate($post_win, $post_lost) {
  include_once('elo-rating-algorithm.php');
  // Get current ratings from DB.
  $current_rating_win = (float) get_post_meta($post_win, 'elo_rating__rating', TRUE);
  if (empty($current_rating_win)) $current_rating_win = 1000;
  $current_rating_lost = (float) get_post_meta($post_lost, 'elo_rating__rating', TRUE);
  if (empty($current_rating_lost)) $current_rating_lost = 1000;
  
  $score_win = 1;
  $score_lost = 0;
  
  // Calculate new ratings
  $handler = new EloRatingAlgorithm($current_rating_win, $current_rating_lost, $score_win, $score_lost );
  
  //Save new ratings.
  $new_ratings = $handler->getNewRatings();
  update_post_meta($post_win, 'elo_rating__rating', $new_ratings[a], $current_rating_win);
  update_post_meta($post_lost, 'elo_rating__rating', $new_ratings[b], $current_rating_lost);
}